<?php
/*
Plugin Name: Markdown on Save Improved
Description: Allows you to compose content in Markdown on a per-item basis from WP admin or mobile/3rd-party apps. The markdown version is stored separately, so you can deactivate this plugin and your posts won't spew out Markdown. Based on <a href="http://wordpress.org/extend/plugins/markdown-on-save/">Mark Jaquith's plugin</a>.
Version: 1.0.2
Author: Matt Wiebe
Author URI: http://somadesign.ca/
*/

class SD_Markdown {
	const PM = '_cws_is_markdown';
	var $instance;

	public function __construct() {
		$this->instance =& $this;
		add_action( 'init', array( $this, 'init' ) );
	}

	public function init() {
		load_plugin_textdomain( 'markdown-on-save', NULL, basename( dirname( __FILE__ ) ) );
		add_filter( 'wp_insert_post_data', array( $this, 'wp_insert_post_data' ), 10, 2 );
		add_action( 'do_meta_boxes', array( $this, 'do_meta_boxes' ), 20, 2 );
		add_filter( 'edit_post_content', array( $this, 'edit_post_content' ), 10, 2 );
		add_filter( 'edit_post_content_filtered', array( $this, 'edit_post_content_filtered' ), 10, 2 );
		add_action( 'xmlrpc_call', array($this, 'xmlrpc_actions') );
	}
	
	public function xmlrpc_actions($xmlrpc_method) {
		if ( 'metaWeblog.getRecentPosts' === $xmlrpc_method ) {
			add_action( 'parse_query', array($this, 'make_filterable'), 10, 1 );
		}
		else if ( 'metaWeblog.getPost' === $xmlrpc_method ) {
			$this->prime_post_cache();
		}
	}
	
	private function prime_post_cache() {
		global $wp_xmlrpc_server;
		$params = $wp_xmlrpc_server->message->params;
		$post_id = array_shift($params);
		// prime the post cache
		if ( $this->is_markdown($post_id) ) {
			$post = get_post($post_id);
			$post->post_content = $post->post_content_filtered;
			wp_cache_delete($post->ID, 'posts');
			wp_cache_add($post->ID, $post, 'posts');
		}
	}
	
	public function make_filterable($wp_query) {
		$wp_query->set( 'suppress_filters', false );
		add_action( 'the_posts', array($this, 'the_posts'), 10, 2 );
	}
	
	public function the_posts($posts, $wp_query) {
		foreach ( $posts as $key => $post ) {
			if ( $this->is_markdown($post->ID) ) {
				$posts[$key]->post_content = $posts[$key]->post_content_filtered;
			}
		}
		return $posts;
	}

	public function wp_insert_post_data( $data, $postarr ) {
		// run once
		remove_filter( 'wp_insert_post_data', array( $this, 'wp_insert_post_data' ), 10, 2 );
		$box_ticked = ( isset( $_POST['cws_using_markdown'] ) && wp_verify_nonce( $_POST['_cws_markdown_nonce'], 'cws-markdown-save' ) );
		$md_comment_inserted = ( false !== stripos($data['post_content'], '<!--markdown-->') );
		if ( $box_ticked || $md_comment_inserted ) {
			$id = $postarr['ID'];
			$data['post_content_filtered'] = $data['post_content'];
			// Do markdown processing
			$data['post_content'] = $this->process($data['post_content'], $id);
			if ( $postarr['ID'] )
				update_post_meta( $postarr['ID'], self::PM, true );
		} else {
			$data['post_content_filtered'] = '';
			if ( $postarr['ID'] )
				delete_post_meta( $postarr['ID'], self::PM );
		}
		return $data;
	}
	
	protected function process($content, $id) {
		require_once( dirname( __FILE__) . '/markdown-extra/markdown-extra.php' );
		// leaving the comment in produces odd Markdown sometimes
		$content = str_ireplace('<!--markdown-->', '', $content);
		// Links with "quoted titles" bork without stripping slashes.
		// Stripslashes would lose desired backslashes. So, regex.
		$content = preg_replace( '/\\\"/', '"', $content );
		// convert to Markdown, but remove <p> tags for wpautop to work later
		$content = $this->unp( Markdown( $content ) );
		// reference the post_id to make footnote ids unique
		$content = preg_replace('/fn(ref)?:/', "fn$1-$id:", $content);
		return $content;
	}

	public function do_meta_boxes( $type, $context ) {
		if ( 'side' == $context && in_array( $type, array_keys( get_post_types() ) ) )
			add_meta_box( 'cws-markdown', __( 'Markdown', 'markdown-on-save' ), array( $this, 'meta_box' ), $type, 'side', 'high' );
	}

	public function meta_box() {
		global $post;
		wp_nonce_field( 'cws-markdown-save', '_cws_markdown_nonce', false, true );
		echo '<p><input type="checkbox" name="cws_using_markdown" id="cws_using_markdown" value="1" ';
		checked( !! get_post_meta( $post->ID, self::PM, true ) );
		echo ' /> <label for="cws_using_markdown">' . __( 'This post is formatted with Markdown', 'markdown-on-save' ) . '</label></p>';
	}

	private function unp( $content ) {
		return preg_replace( "#<p>(.*?)</p>(\n|$)#", '$1$2', $content );
	}

	private function is_markdown( $id ) {
		return !! get_post_meta( $id, self::PM, true );
	}

	public function edit_post_content( $content, $id ) {
		if ( $this->is_markdown( $id ) ) {
			$post = get_post( $id );
			if ( $post )
				$content = $post->post_content_filtered;
		}
		return $content;
	}

	public function edit_post_content_filtered( $content, $id ) {
		return $content;
	}

}

new SD_Markdown;
